package com.zhongshu.reward.server.core.dao.Impl;

import com.zhongshu.reward.client.model.InviteSuccessModel;
import com.zhongshu.reward.client.model.TotalStatisticsModel;
import com.zhongshu.reward.client.type.ReceiptsStatus;
import com.zhongshu.reward.client.type.ReceiptsType;
import com.zhongshu.reward.server.core.dao.WalletReceiptsDao;
import com.zhongshu.reward.server.core.dao.extend.WalletReceiptsDaoExtend;
import com.zhongshu.reward.server.core.domain.WalletReceipts;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.query.criteria.internal.compile.CriteriaCompiler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

import java.util.List;

/**
 * @author wjf
 * @date 2024/8/12
 */
public class WalletReceiptsDaoImpl implements WalletReceiptsDaoExtend {

    @Autowired
    MongoTemplate mongoTemplate;

    @Override
    public List<WalletReceipts> inviteDetail(String inviteUserId, Long startTime, Long endTime) {
        Criteria criteria = new Criteria();
        criteria.andOperator(Criteria.where("createTime").gte(startTime),Criteria.where("createTime").lte(endTime));


        if (StringUtils.isNotEmpty(inviteUserId)){
            criteria.and("inviteUserId").is(inviteUserId);
        }
        Query query = Query.query(criteria);
        query.with(Sort.by(Sort.Order.desc("createTime")));

        return mongoTemplate.find(query, WalletReceipts.class);
    }

    @Override
    public List<WalletReceipts> listMonth(Long startTime, Long endTime, Integer day) {
        Criteria criteria = new Criteria();
        criteria.andOperator(Criteria.where("createTime").gte(startTime),Criteria.where("createTime").lte(endTime));

//        criteria.and("createTime").gte(startTime);
//
//        criteria.and("createTime").lte(endTime);

        criteria.and("receiptsType").is(ReceiptsType.COMMISSION);
        criteria.and("status").is(ReceiptsStatus.WAIT);
//        criteria.and("outTradeNo").in(setMealCodeList);
//        criteria.and("ruler.keep").is(true);
        criteria.and("ruler.day").is(day);
        Query query = Query.query(criteria);
        query.with(Sort.by(Sort.Order.desc("createTime")));

        return mongoTemplate.find(query, WalletReceipts.class);
    }

    @Override
    public List<InviteSuccessModel> myInvite(String userId) {
        Criteria criteria = new Criteria();
        criteria.and("inviteUserId").is(userId);
        criteria.and("status").in(List.of(ReceiptsStatus.WAIT, ReceiptsStatus.RECEIPTS));
        criteria.and("receiptsType").is(ReceiptsType.COMMISSION);

        Aggregation aggregation = Aggregation.newAggregation(
                Aggregation.match(criteria),
//                Aggregation.project("_id","amountTotal"),
                Aggregation.group("userId").sum("total").as("totalAmount"),
                Aggregation.project().and("_id").as("userId").and("totalAmount").as("totalAmount")
//                Aggregation.group("tradeType").last("tradeType").as("_id")
        );


        AggregationResults<InviteSuccessModel> groupList = mongoTemplate.aggregate(aggregation, WalletReceipts.class, InviteSuccessModel.class);
        return groupList.getMappedResults();
    }

    @Override
    public List<WalletReceipts> listMonthBySetMealCode(Long startTime, Long endTime, List<String> setMealCodeList) {
        Criteria criteria = new Criteria();
        criteria.andOperator(Criteria.where("createTime").gte(startTime),Criteria.where("createTime").lte(endTime));

//        criteria.and("createTime").gte(startTime);
//
//        criteria.and("createTime").lte(endTime);

        criteria.and("receiptsType").is(ReceiptsType.COMMISSION);
        criteria.and("status").in(ReceiptsStatus.WAIT,ReceiptsStatus.RECEIPTS);
        criteria.and("outTradeNo").in(setMealCodeList);
        Query query = Query.query(criteria);
        query.with(Sort.by(Sort.Order.asc("createTime")));
        return mongoTemplate.find(query, WalletReceipts.class);
    }
}
